﻿using Microsoft.EntityFrameworkCore;
using PaydayBackend.Models;
using PaydayBackend.Utils;
using UnidecodeSharpFork;

namespace PaydayBackend.Services;

public interface IAdminService
{
    // Bank
    public Task<string> AddBank(string bankName, IFormFile file);
    public Task<IEnumerable<Bank>> GetAllBanks();
    public Task AddLoanTerm(LoanTerm loanTerm);
    public Task<string> RemoveAllLoanTermsByBankId(long bankId);
    public Task<IEnumerable<LoanTerm>?> GetAllLoanTermsByBankId(long bankId);
    // University
    public Task<bool> UniversityIsExist(long universityId);
    public Task<IEnumerable<University>> GetAllUniversity();
    public Task<string> AddUniversity(string shortName, string fullName, IFormFile file);
    public Task<string> AddUniversityDirection(UniversityDirectionDto universityDirection);
}

public class AdminService : IAdminService
{
    private readonly DatabaseContext _databaseContext;
    private readonly IStorageService _storageService;

    private async Task<bool> BankIsExist(long bankId)
    {
        if (await _databaseContext.Banks.Where(x => x.Id == bankId).FirstOrDefaultAsync() == null)
            return false;

        return true;
    }
    
    public async Task<bool> UniversityIsExist(long universityId)
    {
        if (await _databaseContext.Universities.Where(x => x.Id == universityId).FirstOrDefaultAsync() == null)
            return false;

        return true;
    }

    public AdminService(DatabaseContext databaseContext, IStorageService storageService)
    {
        _databaseContext = databaseContext;
        _storageService = storageService;
    }
    
    // -------------------------------------| Банки |-------------------------------------

    public async Task<string> AddBank(string bankName, IFormFile file)
    {
        string sizeValidator = FileHelper.ValidateMaxFileSize(file, 5, new []{".png"});
        if (sizeValidator != "OK")
            return sizeValidator;

        var fileStream = new MemoryStream();
        await file.CopyToAsync(fileStream);
        fileStream.Position = 0;
        string imageValidator = await FileHelper.ValidateImage(fileStream, null, 200, 200);
        if (imageValidator != "OK")
            return imageValidator;
                
        await FileHelper.CropImage(fileStream);

        var filename = bankName.Unidecode().Replace(" ", "-") + ".png";
        if (await _storageService.UploadFile(fileStream, "logos", filename))
        {
            var bank = new Bank();
            bank.Name = bankName;
            bank.ImageUrl = "https://payday.zetcraft.ru/logos/" + filename;
        
            await _databaseContext.Banks.AddAsync(bank);
            await _databaseContext.SaveChangesAsync();
            return "ОК";
        }
        
        return "Image not uploaded";
    }

    public async Task<IEnumerable<Bank>> GetAllBanks()
    {
        return await _databaseContext.Banks.ToListAsync();
    }

    public async Task AddLoanTerm(LoanTerm loanTerm)
    {
        await _databaseContext.LoanTerms.AddAsync(loanTerm);
        await _databaseContext.SaveChangesAsync();
    }

    public async Task<string> RemoveAllLoanTermsByBankId(long bankId)
    {
        if (await BankIsExist(bankId))
            return "Bank not found";
        
        var result = await _databaseContext.LoanTerms.Where(x => x.Bank.Id == bankId).ToListAsync();
        _databaseContext.LoanTerms.RemoveRange(result);

        return "OK";
    }

    public async Task<IEnumerable<LoanTerm>?> GetAllLoanTermsByBankId(long bankId)
    {
        if (await BankIsExist(bankId))
            return null;
        
        return await _databaseContext.LoanTerms.Where(x => x.Bank.Id == bankId).ToListAsync();
    }
    
    // -------------------------------------| Университеты |-------------------------------------

    public async Task<IEnumerable<University>> GetAllUniversity()
    {
        return await _databaseContext.Universities.ToListAsync();
    }
    
    public async Task<string> AddUniversity(string shortName, string fullName, IFormFile file)
    {
        string sizeValidator = FileHelper.ValidateMaxFileSize(file, 5, new []{".png"});
        if (sizeValidator != "OK")
            return sizeValidator;

        var fileStream = new MemoryStream();
        await file.CopyToAsync(fileStream);
        fileStream.Position = 0;
        string imageValidator = await FileHelper.ValidateImage(fileStream, null, 200, 200);
        if (imageValidator != "OK")
            return imageValidator;
                
        await FileHelper.CropImage(fileStream);

        var filename = shortName.Unidecode().Replace(" ", "-") + ".png";
        if (await _storageService.UploadFile(fileStream, "logos", filename))
        {
            var university = new University();
            university.Name = shortName;
            university.FullName = fullName;
            university.ImageUrl = "https://payday.zetcraft.ru/logos/" + filename;
        
            await _databaseContext.Universities.AddAsync(university);
            await _databaseContext.SaveChangesAsync();
            return "ОК";
        }
        
        return "Image not uploaded";
    }

    public async Task<string> AddUniversityDirection(UniversityDirectionDto dto)
    {
        var university = await _databaseContext.Universities.Where(x => x.Id == dto.UniversityId).FirstOrDefaultAsync();
        if (university == null)
            return "University not found";

        var universityDirection = new UniversityDirection();
        universityDirection.UniversityId = dto.UniversityId;
        universityDirection.Name = dto.Name;
        universityDirection.Code = dto.Code;
        universityDirection.Url = dto.Url;
        universityDirection.PlaceCost = dto.PlaceCost;
        
        await _databaseContext.UniversityDirections.AddAsync(universityDirection);
        await _databaseContext.SaveChangesAsync();

        return "OK";
    }
}